unit role;

interface

uses Windows, SysUtils, Graphics, Classes, util;

type
  TDirection = (drUp, drDown, drLeft, drRight);
  
  TRole = class
  private
    FX, fy: Integer; // y
    FDirection: TDirection; // iV
    FBits, FInvBitmap: TBitmap; // ϤγzKϥιϤ
    
    FPlayBackList, FMoveList: TList; // \Ϊʧ@O
  public
    constructor Create;
    destructor Destroy; virtual;
    
    procedure LoadBits; // J⪺ bitmap
    procedure Draw(Canvas: TCanvas); // øs
    procedure Move(Dir: TDirection); // ʨ (IBz, ʪ~)
    
    procedure SavePlayback; // Nʧ@Oܭ
    procedure CleanMoveList; // Mʧ@O
    
    // mΤV    
    property X: Integer read FX write FX;
    property Y: Integer read fy write fy;
    property Direction: TDirection read FDirection write FDirection;
    
    property PlayBackList: TList read FPlayBackList;
  end;
  
implementation

uses mapunit;

constructor TRole.Create;
begin
  inherited Create;
  
  FMoveList := TList.Create;
  FPlayBackList := TList.Create;
  
  FBits := TBitmap.Create;
  
  FInvBitmap := TBitmap.Create;
  FInvBitmap.Width := TILE_WIDTH;
  FInvBitmap.Height := TILE_Height;
  FInvBitmap.Transparent := True;
  FInvBitmap.TransparentColor := RGB(128, 0, 0);
end;

destructor TRole.Destroy;
begin
  FInvBitmap.Free;
  FBits.Free;
  
  FMoveList.Free;
  FPlayBackList.Free;
  
  inherited Destroy;
end;

procedure TRole.LoadBits;
begin
  FBits.LoadFromFile(FN_ROLEBITS); // Ū BMP , t|ӤVϧ
  
  if FBits.Width < TILE_WIDTH * 4 then // |ӤV
    raise Exception.Create('Width of role bits is invalid');

  FInvBitmap.PixelFormat := FBits.PixelFormat; // Τ@ FInvBitmap  FBits 榡
end;

procedure TRole.Draw(Canvas: TCanvas);
begin
  // NnqXϰ FInvBitmap, AeX FInvBitmap
  FInvBitmap.Canvas.CopyRect(Rect(0, 0, TILE_WIDTH, TILE_Height), FBits.Canvas, Rect(Ord(FDirection) * TILE_WIDTH, 0,
    (Ord(FDirection) + 1) * TILE_WIDTH, TILE_Height));

  // ѨM VCL b Windows 98 UzK bug  workaround    
  if IsWindows98 then
    TransparentBlt(Canvas.Handle, FInvBitmap.Handle, FX * TILE_WIDTH, fy * TILE_Height, RGB(128, 0, 0))
  else
    Canvas.Draw(FX * TILE_WIDTH, fy * TILE_Height, FInvBitmap);
end;

// small but useful function, update x & y location of role
procedure MoveAhead(var X, Y: Integer; Dir: TDirection);
begin
  case Dir of
    drUp: Dec(Y);
    drDown: Inc(Y);
    drLeft: Dec(X);
    drRight: Inc(X);
  end;
end;

procedure TRole.Move(Dir: TDirection);
var
  X, Y, MX, MY: Integer;
begin
  X := FX; Y := fy;
  MoveAhead(X, Y, Dir);
  FDirection := Dir;
  
  if (Y < 0) or (X < 0) or (X >= TILE_NUM_X) or (Y >= TILE_NUM_Y)
    then Exit; // WXedF..
  
  if not Map.CanPass[X, Y] then Exit; // ਫa
  
  if (Map.ItemMap[X, Y] <> 0) then // pGnʹLhm~
  begin
    if not Map.CanMove[X, Y] then Exit; // hʭC
    
    MX := X; MY := Y;
    MoveAhead(MX, MY, Dir);
    if (MY < 0) or (MX < 0) or (MX >= TILE_NUM_X) or (MY >= TILE_NUM_Y) then Exit; // iN~d~
    
    // nhLhsmW঳F, ]OiHʪa
    if (Map.ItemMap[MX, MY] <> 0) or (not Map.CanPass[MX, MY]) then Exit;
    
    Map.ItemMap[MX, MY] := Map.ItemMap[X, Y]; // hʹLh
    Map.ItemMap[X, Y] := 0; // ӪaS~F
  end;
  
  // \X, NʤVOU
  FMoveList.Add(Pointer(Dir));
  
  FX := X; // sm
  fy := Y;
end;

procedure TRole.SavePlayback;
var
  I: Integer;
begin
  FPlayBackList.Clear; // NʰO FPlayBackList , HK
  for I := 0 to FMoveList.Count - 1 do
    FPlayBackList.Add(FMoveList[I]);
end;

procedure TRole.CleanMoveList;
begin
  FMoveList.Clear; // ʰOMHKUϥ
end;

end.

